@@ -11,6 +11,8 @@
 #ifndef _OGR_FDW_H
 #define _OGR_FDW_H 1
 
+#define OGR_FDW_RELEASE_NAME "1.1"
+
 /*
  * PostgreSQL
  */
@@ -137,7 +139,7 @@ typedef struct OgrConnection
 	const char* open_options;   /* GDAL open options */
 	OgrUpdateable ds_updateable;
 	OgrUpdateable lyr_updateable;
-	bool lyr_utf8;        /* OGR layer will return UTF8 strings */
+	int char_encoding;   /* Is OGR layer UTF? Has user provided encoding open option? */
 	GDALDatasetH ds;      /* GDAL datasource handle */
 	OGRLayerH lyr;        /* OGR layer handle */
 } OgrConnection;
